﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.Win32;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;

namespace JSEnhancements.JSESettings
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    ///
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the 
    /// IVsPackage interface and uses the registration attributes defined in the framework to 
    /// register itself and its components with the shell.
    /// </summary>
    // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
    // a package.
    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the informations needed to show the this package
    // in the Help/About dialog of Visual Studio.
	[ProvideOptionPageAttribute(typeof(OptionsPage), "JSEnhancements", "Options", 105, 106, true)]
	[ProvideProfileAttribute(typeof(OptionsPage), "JSEnhancements", "Options", 105, 106, true, DescriptionResourceID = 105)]
	// This attribute is needed to let the shell know that this package exposes some menus.
	[ProvideMenuResource("Menus.ctmenu", 1)]
    [Guid(GuidList.guidJSESettingsPkgString)]
    public sealed class JSESettingsPackage : Package
    {
        /// <summary>
        /// Default constructor of the package.
        /// Inside this method you can place any initialization code that does not require 
        /// any Visual Studio service because at this point the package object is created but 
        /// not sited yet inside Visual Studio environment. The place to do all the other 
        /// initialization is the Initialize method.
        /// </summary>
        public JSESettingsPackage()
        {
           
        }

		protected override void Initialize()
		{
			base.Initialize();

			// Add our command handlers for menu (commands must exist in the .vsct file)
			OleMenuCommandService mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
			if (null != mcs)
			{
				// Create the command for the menu item.
				CommandID ctdCommandID = new CommandID(GuidList.guidJSECmdSet, (int)PkgCmdIDs.cmdIDCollapseToDefinitions);
				MenuCommand mnuCtd = new MenuCommand(CtdMenu_Clicked, ctdCommandID);
				mcs.AddCommand(mnuCtd);
			}
		}

		private void CtdMenu_Clicked(object sender, EventArgs e)
		{
			Global.DTE.ExecuteCommand("Edit.CollapsetoDefinitions");
		}
    }
}
